#using LinearAlgebra, MPI, Optim, LineSearches, Pkg
#using VMEC, StellaratorOptimizationMetrics
#Pkg.activate(".")
#using lasso

#BLAS.set_num_threads(Threads.nthreads())

opt = lasso.Optimizer(GradientDescent(alphaguess=1e-5, linesearch=BackTracking()),
                      Optim.Options(iterations=10))
prob = lasso.Problem()
lasso.add_optimizer!(prob, opt)

nml = VMEC.read_vmec_namelist("/ssd1/Wistell/configs/chup/input.chup")

vmec_wrapper = lasso.EquilibriumWrapper{Float64, Vmec}(nml)
vmec = readVmecWout("/ssd1/Wistell/configs/chup/wout_chup.nc")
lasso.update_equilibrium!(vmec_wrapper, vmec)
lasso.add_equilibrium_wrapper!(prob, vmec_wrapper)

vmec_s02 = lasso.@add_derived_geometry!(prob, VmecSurface(0.2, equilibrium = vmec_wrapper))
vmec_s05 = lasso.@add_derived_geometry!(prob, VmecSurface(0.5, equilibrium = vmec_wrapper))
vmec_s08 = lasso.@add_derived_geometry!(prob, VmecSurface(0.8, equilibrium = vmec_wrapper))

lasso.@add_target!(prob, VMEC.quasisymmetry_deviation(1, 4, 16, 16, geometry = vmec_s02), 0.0, 1.0, "qs_vmec(s=0.2)")
lasso.@add_target!(prob, VMEC.quasisymmetry_deviation(1, 4, 16, 16, geometry = vmec_s05), 0.0, 1.0, "qs_vmec(s=0.5)")
lasso.@add_target!(prob, VMEC.quasisymmetry_deviation(1, 4, 16, 16, geometry = vmec_s08), 0.0, 1.0, "qs_vmec(s=0.8)")

ζ_min = 0.0
ζ_max = 100π
Δζ = 2π/100
 
lasso.@add_target!(prob,
                   StellOptMetrics.GammaC.gamma_c_target(geometry = vmec_s02, ζ_min, Δζ, ζ_max,
                                                         100, false, true), 
                   0.0, 1.0, "Γc(s=0.2)")
lasso.@add_target!(prob,
                   StellOptMetrics.GammaC.gamma_c_target(geometry = vmec_s05, ζ_min, Δζ, ζ_max,
                                                         100, false, true), 
                   0.0, 1.0, "Γc(s=0.5)")
lasso.@add_target!(prob,
                   StellOptMetrics.GammaC.gamma_c_target(geometry = vmec_s08, ζ_min, Δζ, ζ_max,
                                                         100, false, true), 
                   0.0, 1.0, "Γc(s=0.8)")

lasso.add_variable!(prob, vmec_wrapper, "rbc(0,1)", finite_diff_step = 0.001)
lasso.add_variable!(prob, vmec_wrapper, "zbs(0,1)", finite_diff_step = 0.001)
lasso.add_variable!(prob, vmec_wrapper, "rbc(1,0)", finite_diff_step = 0.001)
lasso.add_variable!(prob, vmec_wrapper, "zbs(1,0)", finite_diff_step = 0.001)
lasso.add_variable!(prob, vmec_wrapper, "rbc(1,1)", finite_diff_step = 0.001)
lasso.add_variable!(prob, vmec_wrapper, "zbs(1,1)", finite_diff_step = 0.001)

lasso.compute_targets!(prob)
#residual = lasso.optimize_mpi(prob)
