using Optim, PlasmaEquilibriumToolkit, NE3DLE, lasso, LineSearches
#ENV["JULIA_DEBUG"] = lasso

optimizer = lasso.Optimizer(GradientDescent(; alphaguess=1e-2, linesearch=BackTracking()),
                            Optim.Options(iterations = 20))
prob = lasso.Problem()
lasso.add_optimizer!(prob, optimizer)

nml = NE3DLE.read_par_file("/home/bfaber/.julia/dev/NE3DLE/NE3DLE.in")
Ne3dle_wrapper = lasso.EquilibriumWrapper{Float64, NE3DLESurface}(nml)
eq = call_NE3DLE(nml)
lasso.update_equilibrium!(Ne3dle_wrapper, eq)
lasso.add_equilibrium_wrapper!(prob, Ne3dle_wrapper)
Ne3dle_s = lasso.@add_derived_geometry!(prob,NE3DLESurface(equilibrium=Ne3dle_wrapper))
lasso.@add_target!(prob, NE3DLE.quasisymmetry_deviation(geometry=Ne3dle_s), 0.0, 1.0, "qs_ne3dle")
lasso.add_variable!(prob, Ne3dle_wrapper, "Rmn(0,1)", fd_step = 0.01)
lasso.add_variable!(prob, Ne3dle_wrapper, "Rmn(0,-1)", fd_step = 0.01)
lasso.add_variable!(prob, Ne3dle_wrapper, "Rmn(1,0)", fd_step = 0.01)
lasso.add_variable!(prob, Ne3dle_wrapper, "Zmn(0,1)", fd_step = 0.01)
lasso.add_variable!(prob, Ne3dle_wrapper, "Zmn(0,-1)", fd_step = 0.01)
lasso.add_variable!(prob, Ne3dle_wrapper, "Zmn(1,0)", fd_step = 0.01)
#=
lasso.add_variable!(prob, Ne3dle_wrapper, "Rmn(0,2)", fd_step = 0.001)
lasso.add_variable!(prob, Ne3dle_wrapper, "Rmn(0,-2)", fd_step = 0.001)
lasso.add_variable!(prob, Ne3dle_wrapper, "Rmn(2,0)", fd_step = 0.001)
lasso.add_variable!(prob, Ne3dle_wrapper, "Rmn(-2,0)", fd_step = 0.001)
lasso.add_variable!(prob, Ne3dle_wrapper, "Zmn(0,2)", fd_step = 0.001)
lasso.add_variable!(prob, Ne3dle_wrapper, "Zmn(0,-2)", fd_step = 0.001)
lasso.add_variable!(prob, Ne3dle_wrapper, "Zmn(2,0)", fd_step = 0.001)
lasso.add_variable!(prob, Ne3dle_wrapper, "Zmn(-2,0)", fd_step = 0.001)
lasso.add_variable!(prob, Ne3dle_wrapper, "Rmn(1,2)", fd_step = 0.001)
lasso.add_variable!(prob, Ne3dle_wrapper, "Rmn(1,-2)", fd_step = 0.001)
lasso.add_variable!(prob, Ne3dle_wrapper, "Rmn(2,1)", fd_step = 0.001)
lasso.add_variable!(prob, Ne3dle_wrapper, "Rmn(-2,1)", fd_step = 0.001)
lasso.add_variable!(prob, Ne3dle_wrapper, "Zmn(1,2)", fd_step = 0.001)
lasso.add_variable!(prob, Ne3dle_wrapper, "Zmn(1,-2)", fd_step = 0.001)
lasso.add_variable!(prob, Ne3dle_wrapper, "Zmn(2,1)", fd_step = 0.001)
lasso.add_variable!(prob, Ne3dle_wrapper, "Zmn(-2,1)", fd_step = 0.001)
lasso.add_variable!(prob, Ne3dle_wrapper, "Rmn(-1,2)", fd_step = 0.001)
lasso.add_variable!(prob, Ne3dle_wrapper, "Rmn(-1,-2)", fd_step = 0.001)
lasso.add_variable!(prob, Ne3dle_wrapper, "Rmn(2,-1)", fd_step = 0.001)
lasso.add_variable!(prob, Ne3dle_wrapper, "Rmn(-2,-1)", fd_step = 0.001)
lasso.add_variable!(prob, Ne3dle_wrapper, "Zmn(-1,2)", fd_step = 0.001)
lasso.add_variable!(prob, Ne3dle_wrapper, "Zmn(-1,-2)", fd_step = 0.001)
lasso.add_variable!(prob, Ne3dle_wrapper, "Zmn(2,-1)", fd_step = 0.001)
lasso.add_variable!(prob, Ne3dle_wrapper, "Zmn(-2,-1)", fd_step = 0.001)
=#
residual = lasso.optimize_threads(prob)